@@ -2,7 +2,7 @@ package RedisDB;
 
 use warnings;
 use strict;
-our $VERSION = 0.05;
+our $VERSION = 0.06;
 
 use IO::Socket::INET;
 use Socket qw(MSG_DONTWAIT);
@@ -84,14 +84,13 @@ mode and haven't yet got all replies.
 
 sub execute {
     my $self = shift;
-    die "You can't use RedisDB::execute while in pipelining mode."
+    croak "You can't use RedisDB::execute while in pipelining mode."
       if $self->{_commands_in_flight}
           or @{ $self->{_replies} };
+    croak "This function is not available in subscription mode." if $self->{_subscription_loop};
     $_[0] = uc $_[0];
     $self->send_command(@_);
-    my ( $type, $value ) = $self->get_reply;
-    croak $value if $type eq '-';
-    return $value;
+    return $self->get_reply;
 }
 
 # establish connection to the server.
@@ -105,6 +104,7 @@ sub _connect {
         Proto    => 'tcp'
     ) or die "Can't connect to redis server $self->{host}:$self->{port}: $!";
     $self->{_commands_in_flight} = 0;
+    $self->{_subscription_loop}  = 0;
     return 1;
 }
 
@@ -156,7 +156,11 @@ retrieve reply using I<get_reply> method.
 =cut
 
 sub send_command {
-    my $self    = shift;
+    my $self = shift;
+    if ( $self->{_subscription_loop} ) {
+        croak "only (UN)(P)SUBSCRIBE and QUITE allowed in subscription loop"
+          unless $_[0] =~ /^(p?(un)?subscribe|quit)$/i;
+    }
     my $request = _build_redis_request(@_);
     $self->_connect unless $self->{_socket} and $self->{_pid} == $$;
 
@@ -188,42 +192,41 @@ sub reply_ready {
 
 =head2 $self->get_reply
 
-receive reply from the server. Method returns two elements array. First element
-is the character depending on type of reply: "+" one line reply, "-" error, ":"
-integer reply, "$" bulk reply, "*" multi-bulk reply. Second element is reply
-itself, which is scalar for all replies, except that it is array reference for
-multi-bulk.
+receive reply from the server. Method croaks if server returns error reply.
 
 =cut
 
 sub get_reply {
     my $self = shift;
 
-    if ( @{ $self->{_replies} } ) {
-        return @{ shift @{ $self->{_replies} } };
-    }
+    unless ( @{ $self->{_replies} } ) {
+        die "We are not waiting for reply"
+          unless $self->{_commands_in_flight}
+              or $self->{_subscription_loop};
+        die "You can't read reply in child process" unless $self->{_pid} == $$;
+        while ( not $self->_parse_reply ) {
+            my $ret = $self->{_socket}->recv( my $buffer, 4096 );
+            $self->{debug} and warn "Received: $buffer";
+            unless ( defined $ret ) {
+                next if $! == EINTR;
+                die "Error reading reply from server: $!";
+            }
+            if ( $buffer ne '' ) {
 
-    die "We are not waiting for reply" unless $self->{_commands_in_flight};
-    die "You can't read reply in child process" unless $self->{_pid} == $$;
-    while ( not $self->_parse_reply ) {
-        my $ret = $self->{_socket}->recv( my $buffer, 4096 );
-        $self->{debug} and warn "Received: $buffer";
-        unless ( defined $ret ) {
-            next if $! == EINTR;
-            die "Error reading reply from server: $!";
-        }
-        if ( $buffer ne '' ) {
+                # received some data
+                $self->{_buffer} .= $buffer;
+            }
+            else {
 
-            # received some data
-            $self->{_buffer} .= $buffer;
-        }
-        else {
-
-            # disconnected
-            die "Server unexpectedly closed connection before sending full reply";
+                # disconnected
+                die "Server unexpectedly closed connection before sending full reply";
+            }
         }
     }
-    return @{ shift @{ $self->{_replies} } };
+
+    my $res = shift @{ $self->{_replies} };
+    croak $res->[1] if $res->[0] eq '-';
+    return $res->[1];
 }
 
 my @commands = qw(
@@ -288,6 +291,8 @@ transparently restored on sending next command. If you send a command and
 server closed connection without sending complete reply, connection will not be
 restored and module will throw exception.
 
+=cut
+
 =head1 PIPELINING SUPPORT
 
 You can send commands in pipelining mode. In this case you sending multiple
@@ -300,6 +305,228 @@ pipeline mode, you must receive replies on all pipelined commands first.
 
 =cut
 
+=head1 SUBSCRIPTIONS SUPPORT
+
+RedisDB supports subscriptions to redis channels. In subscription mode you can
+subscribe to some channels and receive all messages sent to these channels.
+Every time RedisDB receives message for the channel it invokes callback
+provided by user. User can specify different callbacks for different channels.
+When in subscription mode you can subscribe to additional channels, or
+unsubscribe from channels you subscribed to, but you can't use any other redis
+commands like set, get, etc. Here's example of running in subscription mode:
+
+    my $message_cb = sub {
+        my ($redis, $channel, $pattern, $message) = @_;
+        print "$channel: $message\n";
+    };
+    
+    my $control_cb = sub {
+        my ($redis, $channel, $pattern, $message) = @_;
+        if ($channel eq 'control.quit') {
+            $redis->unsubscribe;
+            $redis->punsubscribe;
+        }
+        elsif ($channel eq 'control.subscribe') {
+            $redis->subscribe($message);
+        }
+    };
+    
+    subscription_loop(
+        subscribe => [ 'news',  ],
+        psubscribe => [ 'control.*' => $control_cb ],
+        default_callback => $message_cb,
+    );
+
+subscription_loop will subscribe you to news channel and control.* channels. It
+will call specified callbacks every time new message received.  You can
+subscribe to additional channels sending their names to control.subscribe
+channel. You can unsubscribe from all channels by sending message to
+control.quit channel. Every callback receives four arguments: RedisDB object,
+channel for which message was received, pattern if you subscribed to this
+channel using I<psubscribe> method, and message itself.
+
+You can publish messages into channels using I<publish> method. This method
+should be called when you in normal mode, and can't be used while you're in
+subscription mode.
+
+Following methods can be used in subscribtion mode:
+
+=cut
+
+=head2 $self->subscription_loop(%parameters)
+
+Enter into subscription mode. Function subscribes you to specified channels,
+waits for messages, and invokes callbacks for every received message. Function
+returns after you unsubscribed from all channels. It accepts following parameters:
+
+=over 4
+
+=item default_callback
+
+reference to the default callback. This callback is invoked for the message if you
+didn't specify other callback for the channel this message comes from.
+
+=item subscribe
+
+array reference. Contains list of channels you want to subscribe. Channel name
+may be optionally followed by reference to callback function for this channel.
+E.g.:
+
+    [ 'news', 'messages', 'errors' => \&error_cb, 'other' ]
+
+channels "news", "messages", and "other" will use default callback, but for
+"errors" channel error_cb function will be used.
+
+=item psubscribe
+
+same as subscribe, but you specify patterns for channels' names.
+
+=back
+
+All parameters are optional, but you must subscribe at least to one channel. Also
+if default_callback is not specified, you have to explicitely specify callback
+for every channel you're going to subscribe.
+
+=cut
+
+sub subscription_loop {
+    my ( $self, %args ) = @_;
+    croak "Already in subscription loop" if $self->{_subscribtion_loop};
+    croak "You can't start subscription loop while in pipelining mode."
+      if $self->{_commands_in_flight}
+          or @{ $self->{_replies} };
+    $self->{_subscribed}        = {};
+    $self->{_psubscribed}       = {};
+    $self->{_subscription_cb}   = $args{default_callback};
+    $self->{_subscription_loop} = 1;
+
+    if ( $args{subscribe} ) {
+        while ( my $channel = shift @{ $args{subscribe} } ) {
+            my $cb;
+            $cb = shift @{ $args{subscribe} } if ref $args{subscribe}[0] eq 'CODE';
+            $self->subscribe( $channel, $cb );
+        }
+    }
+    if ( $args{psubscribe} ) {
+        while ( my $channel = shift @{ $args{psubscribe} } ) {
+            my $cb;
+            $cb = shift @{ $args{psubscribe} } if ref $args{psubscribe}[0] eq 'CODE';
+            $self->psubscribe( $channel, $cb );
+        }
+    }
+    croak "You must subscribe at least to one channel"
+      unless ( keys %{ $self->{_subscribed} }, keys %{ $self->{_psubscribed} } );
+
+    while ( my $msg = $self->get_reply ) {
+        die "Expected multi-bulk reply, but got $msg" unless ref $msg;
+        if ( $msg->[0] eq 'message' ) {
+            $self->{_subscribed}{ $msg->[1] }( $self, $msg->[1], undef, $msg->[2] );
+        }
+        elsif ( $msg->[0] eq 'pmessage' ) {
+            $self->{_psubscribed}{ $msg->[1] }( $self, $msg->[2], $msg->[1], $msg->[3] );
+        }
+        elsif ( $msg->[0] eq 'subscribe' or $msg->[0] eq 'psubscribe' ) {
+
+            # ignore
+        }
+        elsif ( $msg->[0] eq 'unsubscribe' ) {
+            delete $self->{_subscribed}{ $msg->[1] };
+
+            # TODO think about it, not exactly correct
+            last unless $msg->[2] or %{ $self->{_psubscribed} };
+        }
+        elsif ( $msg->[0] eq 'punsubscribe' ) {
+            delete $self->{_psubscribed}{ $msg->[1] };
+            last unless $msg->[2] or %{ $self->{_subscribed} };
+        }
+        else {
+            die "Got unknown reply $msg->[0] in subscription mode";
+        }
+    }
+    $self->_connect;
+    return;
+}
+
+=head2 $self->subscribe($channel[, $callback])
+
+Subscribe to additional I<$channel>. If I<$callback> is not specified, default
+callback will be used.
+
+=cut
+
+sub subscribe {
+    my ( $self, $channel, $callback ) = @_;
+    croak "Must be in subscription loop to subscribe" unless $self->{_subscription_loop};
+    croak "Subscribe to what channel?" unless length $channel;
+    $callback ||= $self->{_subscription_cb}
+      or croak "Callback for $channel not specified, neither default callback defined";
+    $self->{_subscribed}{$channel} = $callback;
+    $self->send_command( "SUBSCRIBE", $channel );
+    return;
+}
+
+=head2 $self->psubscribe($pattern[, $callback])
+
+Subscribe to additional channels matching I<$pattern>. If I<$callback> is not specified, default
+callback will be used.
+
+=cut
+
+sub psubscribe {
+    my ( $self, $channel, $callback ) = @_;
+    croak "Must be in subscription loop to subscribe" unless $self->{_subscription_loop};
+    croak "Subscribe to what channel?" unless length $channel;
+    $callback ||= $self->{_subscription_cb}
+      or croak "Callback for $channel not specified, neither default callback defined";
+    $self->{_psubscribed}{$channel} = $callback;
+    $self->send_command( "PSUBSCRIBE", $channel );
+    return;
+}
+
+=head2 $self->unsubscribe([@channels])
+
+Unsubscribe from the listed I<@channels>. If no channels specified, unsubscribe
+from all channels.
+
+=cut
+
+sub unsubscribe {
+    my $self = shift;
+    return $self->send_command( "UNSUBSCRIBE", @_ );
+}
+
+=head2 $self->punsubscribe([@patterns])
+
+Unsubscribe from the listed I<@patterns>. If no patterns specified, unsubscribe
+from all channels to which you subscribed using I<psubscribe>.
+
+=cut
+
+sub punsubscribe {
+    my $self = shift;
+    return $self->send_command( "PUNSUBSCRIBE", @_ );
+}
+
+=head2 $self->subscribed
+
+Return list of channels to which you have subscribed using I<subscribe>
+
+=cut
+
+sub subscribed {
+    return keys %{ shift->{_subscribed} };
+}
+
+=head2 $self->psubscribed
+
+Return list of channels to which you have subscribed using I<psubscribe>
+
+=cut
+
+sub psubscribed {
+    return keys %{ shift->{_psubscribed} };
+}
+
 # build_redis_request($command, @arguments)
 #
 # Builds unified redis request from given I<$command> and I<@arguments>.
@@ -360,8 +587,20 @@ sub _parse_reply {
         elsif ( $self->{_parse_state} == $READ_NUMBER ) {
             if ( defined( my $line = $self->_read_line ) ) {
                 die "Received invalid integer reply :$line" unless $line =~ /^-?[0-9]+$/;
-                $self->{_parse_reply}[1] = $line;
-                return $self->_reply_completed;
+                if ( $self->{_parse_reply}[0] eq ':' ) {
+                    $self->{_parse_reply}[1] = $line;
+                    return $self->_reply_completed;
+                }
+                else {
+                    push @{ $self->{_parse_reply}[1] }, $line;
+                    if ( --$self->{_parse_mblk_len} ) {
+                        $self->{_parse_state} = $WAIT_BUCKS;
+                        $repeat = 1;
+                    }
+                    else {
+                        return $self->_reply_completed;
+                    }
+                }
             }
         }
         elsif ( $self->{_parse_state} == $READ_BULK_LEN ) {
@@ -430,9 +669,17 @@ sub _parse_reply {
             }
         }
         elsif ( $self->{_parse_state} == $WAIT_BUCKS ) {
-            die "Invalid multi-bulk reply. Expected '\$'"
-              unless substr( $self->{_buffer}, 0, 1, '' ) eq '$';
-            $self->{_parse_state} = $READ_BULK_LEN;
+            my $char = substr( $self->{_buffer}, 0, 1, '' );
+            if ( $char eq '$' ) {
+                $self->{_parse_state} = $READ_BULK_LEN;
+            }
+            elsif ( $char eq ':' ) {
+                $self->{_parse_state} = $READ_NUMBER;
+            }
+            else {
+                die "Invalid multi-bulk reply. Expected '\$' or ':' but got $char"
+                  ;    # $self->{_buffer}";
+            }
             $repeat = 1;
         }
     }
@@ -492,10 +739,6 @@ Test all commands
 
 =item *
 
-User defined error callback
-
-=item *
-
 Handle cases when client is not interested in replies
 
 =item *
@@ -506,10 +749,6 @@ Transactions support (MULTI, EXEC, DISCARD, WATCH, UNWATCH)
 
 Subscriptions support (PSUBSCRIBE, PUNSUBSCRIBE, SUBSCRIBE, UNSUBSCRIBE)
 
-=item *
-
-MONITOR support
-
 =back
 
 =head1 BUGS
